import os
import json
import logging
from flask import Flask, jsonify, request, abort

from p2.store.client import Client
from p2.logging import HttpLogger

app = Flask(__name__)
app.config.from_object('p2.ui.experiments.settings')
app.secret_key = app.config['SECRET_KEY']

log = HttpLogger(
    level = logging.DEBUG,
    silent = False,
    name='p2.ui.experiments'
)
logger = log.getLogger()

p2_store_url = os.getenv('P2_STORE_URL')
if p2_store_url is None:
    raise Exception("No P2_STORE_URL envar found")

client = Client(p2_store_url)

@app.route('/api/display_experiments')
def display_experiments():
    recs = client.select(
        table='experiments',
        fetch=[
            'filename',
            'description',
            'type',
            'creation_time',
            'last_update_time',
            'tag'],
        sort='filename'
    )
    if recs is None:
        logger.error("No experiments found")
        return jsonify([])
    else:
        return jsonify(recs)


@app.route('/api/get_experiment_details')
def get_experiment_details():
    filename = request.args.get('filename')
    type = request.args.get('type')
    logger.debug(f"Getting details for exp. {filename} of type {type}")
    exp = client.select(
        table='experiments',
        filter={'filename': filename, 'type': type},
        fetch=[
            'profile',
            'data'
        ],
        limit=1
    )
    if (exp is None or len(exp) != 1):
        logger.error(f"Could not retrieve experiment {filename}")
        abort(404)
    else:
        exp[0]['profile'] = json.dumps(exp[0]['profile'], indent=2, sort_keys=True)
        if type == 'template':
            exp[0]['data'] = exp[0]['data']
        else:
            exp[0]['data'] = json.dumps(exp[0]['data'], indent=2, sort_keys=True)
    try:
        sessions = client.select(
            table='exp_session',
            filter={'filepath': filename},
            sort='start_time desc',
            fetch=[
                'exp_id',
                'schedule',
                'start_time',
                'end_time',
                'status'
            ])
        if (sessions is not None and len(sessions) > 0):
            exp[0]['sessions'] = """
    <div class="container">
    <div class="table">
    <div class="table-header">
        <div class="header__item"><a id="exp_id" class="filter__link" href="#">Session ID</a></div>
        <div class="header__item"><a id="schedule" class="filter__link filter__link"
        href="#">Schedule</a></div>
        <div class="header__item"><a id="start_time" class="filter__link filter__link"
        href="#">Start time</a></div>
        <div class="header__item"><a id="end_time" class="filter__link filter__link"
        href="#">End time</a></div>
        <div class="header__item"><a id="status" class="filter__link filter__link"
        href="#">Status</a></div>
    </div>
    <div class="table-content">
    """

            for session in sessions:
                if 'start_time' in session and session['start_time'] is not None:
                    session['start_time'] = session['start_time'][:-7]
                else:
                    session['start_time'] = ""
                if 'end_time' in session and session['end_time'] is not None:
                    session['end_time'] = session['end_time'][:-7]
                else:
                    session['end_time'] = ""
                exp[0]['sessions'] += '<div class="table-row">'
                for k in ( 'exp_id', 'schedule', 'start_time', 'end_time', 'status'):
                    exp[0]['sessions'] += f"<div class=\"table-data\">{session.get(k, '')}</div>"
                exp[0]['sessions'] += '</div>'
            exp[0]['sessions'] += """
        </div>
    </div>
    </div>
    """
    except Exception as e:
        logger.error(f"{e}")
        return None

    return jsonify(exp[0])

